﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Windows.Forms;
using System.Drawing.Imaging;
using System.Runtime.InteropServices;

namespace Pseudocolor_image_processing
{
    public partial class Demo : Form
    {
        public Demo()
        {
            InitializeComponent();
            ofd = new OpenFileDialog();
            sfd = new SaveFileDialog();

            string filter = "Image | *.jpg; *.png; *.gif";
            ofd.Filter = filter;
            sfd.Filter = filter;
        }

        OpenFileDialog ofd;
        SaveFileDialog sfd;
        string image_path;
        Bitmap image, processed, display;

        private void colorizeToolStripMenuItem_Click(object sender, EventArgs e)
        {
            processed = processed.Pseudocolorize();
            display = processed.ScaleImage(image_frame.Width, image_frame.Height);
            image_frame.Image = display;
        }

        private void saveImageToolStripMenuItem_Click(object sender, EventArgs e)
        {
            sfd.FileName = image_path;
            if (sfd.ShowDialog() == DialogResult.OK)
            {
                processed.Save(sfd.FileName);
            }
        }

        private void importImageToolStripMenuItem_Click(object sender, EventArgs e)
        {
            if (ofd.ShowDialog() == DialogResult.OK)
            {
                image_path = ofd.FileName;
                image = (Bitmap)Bitmap.FromFile(image_path);
                processed = image;
                display = image.ScaleImage(image_frame.Width, image_frame.Height);
                image_frame.Image = display;
            }
        }
    }

    public static class Ext
    {
        public static Bitmap ScaleImage(this Bitmap image, int width, int height)
        {
            float xratio = (float)width / image.Width;
            float yratio = (float)height / image.Height;
            float ratio = Math.Min(xratio, yratio);

            int w = (int)(image.Width * ratio);
            int h = (int)(image.Height * ratio);

            Bitmap result = new Bitmap(image, new Size(w, h));

            return result;
        }

        public static Bitmap Pseudocolorize(this Bitmap image, int colors = 0)
        {
            int w = image.Width;
            int h = image.Height;

            BitmapData image_data = image.LockBits(
                new Rectangle(0, 0, w, h),
                ImageLockMode.ReadOnly,
                PixelFormat.Format24bppRgb);

            int bytes = image_data.Stride * image_data.Height;
            byte[] buffer = new byte[bytes];
            Marshal.Copy(image_data.Scan0, buffer, 0, bytes);
            image.UnlockBits(image_data);

            byte[] unique = buffer.Distinct().ToArray();
            Array.Sort(unique);

            int blue = 255;
            int green = 0;
            int red = 0;

            byte[][] color_scale = new byte[unique.Length][];
            for (int i = 0; i < unique.Length; i++)
            {
                color_scale[i] = new byte[3];
                
                byte step = (byte)Math.Round(512f / unique.Length);
                if (blue > 0 && green < 255)
                {
                    color_scale[i][0] = (byte)blue;
                    color_scale[i][1] = (byte)green;
                    color_scale[i][2] = 0;

                    blue -= step;
                    green += step;

                    if (blue < 0)
                    {
                        blue = 0;
                    }

                    if (green > 255)
                    {
                        green = 255;
                    }
                }
                else
                {
                    color_scale[i][0] = 0;
                    color_scale[i][1] = (byte)green;
                    color_scale[i][2] = (byte)red;

                    green -= step;
                    red += step;

                    if (green < 0)
                    {
                        green = 0;
                    }

                    if (red > 255)
                    {
                        red = 255;
                    }
                }
            }

            byte[] result = new byte[bytes];
            for (int i = 0; i < bytes; i+=3)
            {
                for (int j = 0; j < 3; j++)
                {
                    result[i + j] = color_scale[Array.IndexOf(unique, buffer[i + j])][j];
                }
            }

            Bitmap result_image = new Bitmap(w, h);
            BitmapData result_data = result_image.LockBits(
                new Rectangle(0, 0, w, h),
                ImageLockMode.WriteOnly,
                PixelFormat.Format24bppRgb);
            Marshal.Copy(result, 0, result_data.Scan0, bytes);
            result_image.UnlockBits(result_data);

            return result_image;
        }
    }
}
